package com.xpo.ltl.api.client;

import com.xpo.ltl.api.client.common.ClientCredentials;
import com.xpo.ltl.api.client.exception.ServiceException;

public abstract class ServiceClient {

	protected String host;
	protected String basePath;
	protected ClientCredentials credentials;
	protected Token token;
	protected int type;

	protected ServiceClient() {
	}

	protected static <T extends ServiceClient> T newInstance(String host, String basePath, ClientCredentials credentials, T newInstance) throws ServiceException {
		T instance;
		try {
			instance = newInstance;
			instance.host = host;
			instance.basePath = basePath;
			instance.credentials = credentials;
			instance.login();
			return instance;
		} catch (Exception e) {
			throw new ServiceException("", e.toString());
		}
	}

	protected static <T extends ServiceClient> T newInstance(String host, String basePath, String accessToken, T newInstance) throws ServiceException {
		T instance;
		try {
			instance = newInstance;
			instance.host = host;
			instance.basePath = basePath;
			instance.setAccessToken(accessToken);
			return instance;
		} catch (Exception e) {
			throw new ServiceException("", e.toString());
		}
	}
	
	// For backward compatibility
	protected static <T extends ServiceClient> T newInstance(String host, String basePath, ClientCredentials credentials, Class<T> type) throws ServiceException {
		T instance;
		try {
			instance = type.newInstance();
			instance.host = host;
			instance.basePath = basePath;
			instance.credentials = credentials;
			instance.login();
			return instance;
		} catch (Exception e) {
			throw new ServiceException("", e.toString());
		}
	}

	// For backward compatibility
	protected static <T extends ServiceClient> T newInstance(String host, String basePath, String accessToken, Class<T> type) throws ServiceException {
		T instance;
		try {
			instance = type.newInstance();
			instance.host = host;
			instance.basePath = basePath;
			instance.setAccessToken(accessToken);
			return instance;
		} catch (Exception e) {
			throw new ServiceException("", e.toString());
		}
	}
	
	private void login() throws ServiceException {
		Resource<Token> resource = new Resource<Token>(host, "", "token", null, Token.class);
		token = resource.getToken(credentials);
		token.setExpiryTimestamp(System.currentTimeMillis() + token.getExpires_in() * 1000);
	}

	public void refreshToken() throws ServiceException {
		Resource<Token> resource = new Resource<Token>(host, "", "token", null, Token.class);
		token = resource.refreshToken(token.getRefresh_token());
		token.setExpiryTimestamp(System.currentTimeMillis() + token.getExpires_in() * 1000);
	}

	protected String getToken() throws ServiceException {
		if (token == null || token.isExpired()) {
			//refreshToken();
			login();
		}
		return token.getAccess_token();
	}

	protected void setAccessToken(String accessToken) {
		token = new Token();
		token.setAccess_token(accessToken);
		token.setExpiryTimestamp(System.currentTimeMillis() + 86400000l);
	}
}
